#include <allegro.h>
#include "Player.h"
#include "Global.h"
#include <linklist.h>
#include "Sounds.h"
#include <tdgui.h>
#include <jrndglob.h>
#include <fstream.h>

//Declare globals
int game_speed, scrx, scry, crater_rate, craters;

void PlayGame(LinkList<Player>& players);
//MainMenu returns true if the game should proceed else false
bool MainMenu(LinkList<Player>& players);

//Local functions
const int GUI_MODE = 0;
const int GAME_MODE = 1;
bool SetGfxMode(int mode); //true on error
void LoadConfig(const char* filename, LinkList<Player>& players);
void SaveConfig(const char* filename, LinkList<Player>& players);
void PurgePlayers(LinkList<Player>& players, LinkList<Player>& inactives);
void MergePlayers(LinkList<Player>& players, LinkList<Player>& inactives);

int main(int argc, char* argv[]) {
  randomize();
  allegro_init();
  if (SetGfxMode(GUI_MODE))
    return 1;

  if (install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL)) {
    set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
    allegro_message("Cannot init sound: %s", allegro_error);
    return 1;
  }

  if (LoadSounds())
    return 1;
  install_keyboard();
  install_joystick(JOY_TYPE_AUTODETECT);
  install_timer();
  install_mouse();
  bill_init();

  LinkList<Player> players;
  LinkList<Player> inactives;
  LoadConfig("config.dat", players);

  while (MainMenu(players)) {
    //Set the new gfx mode
    PurgePlayers(players, inactives);
    if (players.Length() > 1) {
      //Only if there are players to play!
      if (SetGfxMode(GAME_MODE))
        return 1;
      PlayGame(players);
      if (SetGfxMode(GUI_MODE))
        return 1;
    }
    MergePlayers(players, inactives);
  }

  SaveConfig("config.dat", players);

  PlaySound(sucks);
  rest(2000); //to let sound play
  UnloadSounds();
  allegro_exit();
  return 0;
} END_OF_MAIN();

//true on error
bool SetGfxMode(int mode) {
  if (mode == GUI_MODE) {
    set_color_depth(16);
    if (set_gfx_mode(GFX_AUTODETECT, GUI_X, GUI_Y, 0, 0) == -1) {
      set_color_depth(15);
      if (set_gfx_mode(GFX_AUTODETECT, GUI_X, GUI_Y, 0, 0) == -1) {
        allegro_message("Graphics error: %s", allegro_error);
        return true;
      }
    }
    return false;
  } else if (mode == GAME_MODE) {
    set_color_depth(16);
    if (set_gfx_mode(GFX_AUTODETECT, scrx, scry, 0, 0) == -1) {
      set_color_depth(15);
      if (set_gfx_mode(GFX_AUTODETECT, scrx, scry, 0, 0) == -1) {
        set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
        allegro_message("Graphics error: %s", allegro_error);
        return true;
      }
    }
    return false;
  }
  return true;
}

void LoadConfig(const char* filename, LinkList<Player>& players) {
  //Load player/config data
  ifstream infile(filename, ios::binary | ios::nocreate);
  if (!infile.fail()) {
    //File is good
    int numPlayers = 0;
    infile.read((char*)&game_speed, sizeof(game_speed));
    infile.read((char*)&scrx, sizeof(scrx));
    infile.read((char*)&scry, sizeof(scry));
    infile.read((char*)&craters, sizeof(craters));
    infile.read((char*)&crater_rate, sizeof(crater_rate));
    infile.read((char*)&numPlayers, sizeof(numPlayers));
    for (int c=0; c<numPlayers; c++) {
      Player* temp = new Player();
      temp->Load(infile);
      players._MoveData(temp);
    }
  } else {
    //I/O error (probably non-existant file)
    game_speed = 5;
    scrx = 800;
    scry = 600;
    craters = 1;
    crater_rate = 75;
  }
  infile.close();
}

void SaveConfig(const char* filename, LinkList<Player>& players) {
  //Save player/config data
  ofstream outfile(filename, ios::binary | ios::out);
  outfile.write((char*)&game_speed, sizeof(game_speed));
  outfile.write((char*)&scrx, sizeof(scrx));
  outfile.write((char*)&scry, sizeof(scry));
  outfile.write((char*)&craters, sizeof(craters));
  outfile.write((char*)&crater_rate, sizeof(crater_rate));
  int numPlayers = players.Length();
  outfile.write((char*)&numPlayers, sizeof(numPlayers));
  for (int c=0; c<players.Length(); c++)
    players.GetItem(c)->Serialize(outfile);
  outfile.close();
}

void PurgePlayers(LinkList<Player>& players, LinkList<Player>& inactives) {
  players.SetToStart();
  int l = players.Length();
  for (int c=0; c < l; c++) {
    Player* temp = players._GetNext();
    if (!temp->IsActive()) {
      //COPY to inactives list, so _deletethis doesn't mess us up
      inactives._MoveData(new Player(*temp));
      players._DeleteThis();
    } else {
      players._Advance();
    }
  }
}

void MergePlayers(LinkList<Player>& players, LinkList<Player>& inactives) {
  players.SetToStart();
  int l = players.Length();
  for (int c=0; c < l; c++) {
    //Move active players into inactive, so they will be "on top"
    inactives._MoveData(new Player(*players._GetNext()));
    players._DeleteThis();
  }
  players.Swap(inactives); //Switch the lists
}