#include "Player.h"
#include <allegro.h>
#include <jrnd.h>
#include <string.h>
#include <fstream.h>
#include "Sounds.h"

Player::Player() {
  name = strdup("Player");
  col = makecol(getrnd(256), getrnd(256), getrnd(256));
  sound = beep;
  input = KEYBOARD;
  c_up = KEY_UP;
  c_left = KEY_LEFT;
  c_right = KEY_RIGHT;
  c_down = KEY_DOWN;
  dead = false;
  active = true;
  x = y = 5;
  dir = RIGHT;
  wins = losses = 0;
}

Player::Player(Player& rhs) {
  name = strdup(rhs.name);
  col = rhs.col;
  sound = rhs.sound;
  input = rhs.input;
  c_up = rhs.c_up;
  c_left = rhs.c_left;
  c_right = rhs.c_right;
  c_down = rhs.c_down;
  dead = rhs.dead;
  active = rhs.active;
  x = rhs.x;
  y = rhs.y;
  dir = rhs.dir;
  wins = rhs.wins;
  losses = rhs.losses;
}

Player::Player(char* name2, int col2, int sound2, int control, int up, int left, int right,
               int down) : x(5), y(5), dir(RIGHT), col(col2), c_up(up),
               c_left(left), c_right(right), c_down(down), input(control),
               dead(false), active(true), sound(sound2), wins(0), losses(0), name(NULL) {
  if (name2)
    name = strdup(name2);
  else
    name = strdup("Player");
}

Player::~Player() {
  if (name)
    delete[] name;
}

void Player::Serialize(ofstream& file) {
  int namelen = strlen(name);
  file.write((char*)&namelen, sizeof(namelen));
  file.write(name, namelen);
  file.write((char*)&col, sizeof(col));
  file.write((char*)&sound, sizeof(sound));
  file.write((char*)&input, sizeof(input));
  file.write((char*)&c_up, sizeof(c_up));
  file.write((char*)&c_left, sizeof(c_left));
  file.write((char*)&c_right, sizeof(c_right));
  file.write((char*)&c_down, sizeof(c_down));
  file.write((char*)&active, sizeof(active));
  file.write((char*)&wins, sizeof(wins));
  file.write((char*)&losses, sizeof(losses));;
}

//Returns true on loading error
bool Player::Load(ifstream& file) {
  int namelen = 0;
  file.read((char*)&namelen, sizeof(namelen));
  name = new char[namelen + 1];
  file.read(name, namelen);
  name[namelen] = '\0'; //NULL terminate string

  file.read((char*)&col, sizeof(col));
  file.read((char*)&sound, sizeof(sound));
  file.read((char*)&input, sizeof(input));
  file.read((char*)&c_up, sizeof(c_up));
  file.read((char*)&c_left, sizeof(c_left));
  file.read((char*)&c_right, sizeof(c_right));
  file.read((char*)&c_down, sizeof(c_down));
  file.read((char*)&active, sizeof(active));
  file.read((char*)&wins, sizeof(wins));
  file.read((char*)&losses, sizeof(losses));;
  dead = false;
  x = y = 5;
  dir = RIGHT;
  return false;
}

void Player::SetPos(int x2, int y2, int dir2) {
  x = x2;
  y = y2;
  dir = dir2;
}

void Player::Reset() {
  dead = false;
}

bool Player::IsDead() {
  return dead;
}

char* Player::GetName() {
  return name;
}

void Player::SetName(char* name2) {
  if (name)
    delete[] name;
  name = strdup(name2);
}

void Player::GetColors(int& red, int& green, int& blue) {
  red = getr(col);
  green = getg(col);
  blue = getb(col);
}

void Player::SetColor(int col2) {
  col = col2;
}

bool Player::IsActive() {
  return active;
}

void Player::ToggleActive() {
  active = !active;
}

void Player::Draw() {
  acquire_screen();
  putpixel(screen, x, y, col);
  release_screen();
}

void Player::Update() {
  //Get controls
  if (input == 0) {
    //Keyboard input
    if (key[c_up]) {
      if ((dir != DOWN) && (dir != UP)) {
        dir = UP;
        PlaySound(sound);
      }
    } else if (key[c_down]) {
      if ((dir != UP) && (dir != DOWN)) {
        dir = DOWN;
        PlaySound(sound);
      }
    } else if (key[c_right]) {
      if ((dir != LEFT) && (dir != RIGHT)) {
        dir = RIGHT;
        PlaySound(sound);
      }
    } else if (key[c_left]) {
      if ((dir != RIGHT) && (dir != LEFT)) {
        dir = LEFT;
        PlaySound(sound);
      }
    }
  } else {
    //Joystick input
    if (joy[input-1].stick[0].axis[1].d1) {
      if ((dir != DOWN) && (dir != UP)) {
        dir = UP;
        PlaySound(sound);
      }
    } else if (joy[input-1].stick[0].axis[1].d2) {
      if ((dir != DOWN) && (dir != UP)) {
        dir = DOWN;
        PlaySound(sound);
      }
    } else if (joy[input-1].stick[0].axis[0].d2) {
      if ((dir != RIGHT) && (dir != LEFT)) {
        dir = RIGHT;
        PlaySound(sound);
      }
    } else if (joy[input-1].stick[0].axis[0].d1) {
      if ((dir != RIGHT) && (dir != LEFT)) {
        dir = LEFT;
        PlaySound(sound);
      }
    }
  }
}

bool Player::Calc() {
  //calc positions
  switch (dir) {
  case UP:
    y--;
    break;
  case DOWN:
    y++;
    break;
  case RIGHT:
    x++;
    break;
  case LEFT:
    x--;
    break;
  }
  
  //detect collision
  acquire_screen();
  dead = ((x<0) || (x>SCREEN_W-1) || (y<0) || (y>SCREEN_H-1) || getpixel(screen, x, y) != 0);
  release_screen();
 
  if (dead)
    losses++;
  return dead;
}

void Player::Win() {
  wins++;
}

int Player::GetWins() {
  return wins;
}

int Player::GetLosses() {
  return losses;
}

int  Player::GetSound() {
  return sound;
}

void Player::SetSound(int sound2) {
  sound = sound2;
}

int  Player::GetInput() {
  return input;
}

void Player::SetInput(int input2) {
  input = input2;
}

void Player::GetControls(int& up, int& left, int& right, int& down) {
  up = c_up;
  left = c_left;
  right = c_right;
  down = c_down;
}

void Player::SetControls(int up, int left, int right, int down) {
  c_up = up;
  c_left = left;
  c_down = down;
  c_right = right;
}

void Player::ResetScores() {
  wins = losses = 0;
}

Player& Player::operator = (Player& rhs) {
  if (name)
    delete[] name;
  name = strdup(rhs.name);
  col = rhs.col;
  sound = rhs.sound;
  input = rhs.input;
  c_up = rhs.c_up;
  c_left = rhs.c_left;
  c_right = rhs.c_right;
  c_down = rhs.c_down;
  x = rhs.x;
  y = rhs.y;
  dir = rhs.dir;
  wins = rhs.wins;
  losses = rhs.losses;

  return *this;
}